// IGOR - a birthday-reminder

/*  Copyright (c) 2005-2006 "Meillo  r e t u r n s"

    Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
    documentation files (the "Software"), to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
    and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all copies or substantial portions
    of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
    TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
    CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

*/

/*  Additional info about the project:
      Author: Markus Schnalke  known as "Meillo  r e t u r n s"
      Website: http://prog.marmaro.de
      Started: 2005-12-30
      Release v0.2: 2006-01-19
*/

import java.io.*;
import java.util.*;

public class Igor {
  // consts
  public final String VERSION = "Igor 0.2 (the java-version)";

  // vars
  private String language = "de";
  private String filename = "days.txt";    /// TODO: cmdline-parameter
  private int daysAhead = 14;          /// TODO: commandline-parameter
  private GregorianCalendar calendarNowH = new GregorianCalendar(); // (YYYY, MM, DD) for testing
  private GregorianCalendar calendarNow = new GregorianCalendar(calendarNowH.get(GregorianCalendar.YEAR),
                                                                calendarNowH.get(GregorianCalendar.MONTH),
                                                                calendarNowH.get(GregorianCalendar.DATE));
  private long dateNow = calendarNow.getTimeInMillis() / 86400000; // amount of days since "the beginning"
  private int yearNow = calendarNow.get(GregorianCalendar.YEAR); // the year now

  // constructor --------------------------------------------------------------------
  public Igor(String[] args) {
    if (args.length <= 0) {
      // about
      String[] about = {
        "",
        "------------------------------",
        "| IGOR - a birthday reminder |",
        "------------------------------",
        "",
        "Copyright 2005/2006  by 'Meillo  r e t u r n s'",
        "This program is distributed under the terms of the Expat-License",
        "Feel free to download it from http://prog.marmaro.de",
        "",
        "",
        "Usage:",
        "",
        "  'java Igor <MODE> [language]'",
        "",
        "  <MODE>     required  (or you'll see this page again)",
        "  [language] optional  either 'de' or 'en', default is 'de'",
        "                       this affects only '-o' and '-g'",
        "",
        "  '-o'         for commandline-output",
        "  '-g'         for a graphical output-window",
        ///"  '-e'         for graphical inputfile-editing",             /// TODO
        "  '--example'  prints an example of the inputfile",
        "  '-V'         displays the programm-version",
        "",
        "Example: 'java -jar Igor.jar -o en'",
        "",
        "",
      };

      for (int i = 0; i < about.length; i++) {
        System.out.println(about[i]);
      }

    } else {                  /// TODO: better commandline-parameter-algo (should work unix-like)
///*
      if (args[0].equals("-o")) {  // Comandline-Output (-o)
        // change language
        if (args.length >= 2) {
          this.language = args[1];
        }
        Dataset[] datasets = selectRelevantDatasets(readInput());
        if (datasets.length > 0) {
          // print the data to the console
          String[] data = formatOutput(datasets);
          for (int i = 0; i < data.length; i++) {
            System.out.println(data[i]);
          }
        }

      } else if (args[0].equals("-g")) { // GUI-output (-g)
        // change language
        if (args.length >= 2) {
          this.language = args[1];
        }
        FormIgorOutput form = new FormIgorOutput(this);
        form.setVisible(true);
                                                                     /// TODO
/*      } else if (args[0].equals("-e")) { // Edit-GUI (-e)
        FormIgorEdit form = new FormIgorEdit(this);
        form.setVisible(true);
*/
      } else if (args[0].equals("-V")) { // version-output (-V)
        System.out.println(VERSION);

      } else if (args[0].equals("--example")) { // example inputfile (--example)
        String[] about = {
          "",
          "The inputfile for Igor is by default 'days.txt',which",
          "is located in the same directory as the program itself.",
          "",
          "Here is an example file",
          "---- days.txt ----------",
          "1941-05-24 Birthday of Bob Dylan",
          "1930-08-25 Birthday of Sean Connery",
          "1943-12-31 Birthday of John Denver",
          "1942-01-18 Birthday of Muhammad Ali",
          "1935-12-01 Birthday of Woody Allen",
          "---- EOF ---------------",
          "",
        };

        for (int i = 0; i < about.length; i++) {
          System.out.println(about[i]);
        }

      } else {
        System.out.println("EE: Invalid commandline-parameter");
      }
//*/
    }
  }



  // main ------------------------------------------------------------------------------
  public static void main(String[] args) {
    new Igor(args);
  }



  // methods ----------------------------------------------------------------------------

  Dataset[] readInput() {
    // read data from file
    Vector inputLines = new Vector();

    try {
      BufferedReader fin = new BufferedReader(new FileReader(filename));

      String line;
      while ((line = fin.readLine()) != null) {
        // allow empty lines and comments with # at the beginning of the line
        if (!line.trim().equals("") && !line.trim().substring(0,1).equals("#")) {
          inputLines.add(line);
        }
      }
      fin.close();
    } catch (FileNotFoundException e) {
      System.out.println("EE: the inputfile (default: 'days.txt') was not found\n" +
                           "see parameter '--example' for help");
    } catch (IOException e) {
      System.err.println(e);
    }

    Dataset[] inputDatasets = new Dataset[inputLines.size()];
    for (int i = 0; i < inputLines.size(); i++) {
      inputDatasets[i] = new Dataset((String) inputLines.get(i));
    }
    return inputDatasets;
  }


  Dataset[] selectRelevantDatasets(Dataset[] datasets) {
    Vector relevantDatasetsVec = new Vector();

    // remove all datasets that are not in the relevant timesector
    for (int i = 0; i < datasets.length; i++) {
      long dateDataset = new GregorianCalendar(
                               yearNow, datasets[i].getMonthInt()-1, datasets[i].getDayInt()
                               ).getTimeInMillis() / 86400000;
      if ((dateDataset >= dateNow && dateDataset <= dateNow + daysAhead) ||
          (dateDataset+365 >= dateNow && dateDataset+365 <= dateNow + daysAhead)) {
         // this dataset is relevant
         relevantDatasetsVec.add(datasets[i]);
      }
    }

    // converting Vector 2 Dataset[]
    Dataset[] relevantDatasets = new Dataset[relevantDatasetsVec.size()];
    for (int i = 0; i < relevantDatasetsVec.size(); i++) {
      relevantDatasets[i] = (Dataset) relevantDatasetsVec.get(i);
    }

    return relevantDatasets;
  }


  String[] formatOutput(Dataset[] datasets) {
    // convert the datasets to strings and do the formating
    String[] returnData = new String[datasets.length];
    int[] datasetOrder = new int[datasets.length];

    for (int i = 0; i < datasets.length; i++) {

      long dateDataset = new GregorianCalendar(
                              yearNow, datasets[i].getMonthInt()-1, datasets[i].getDayInt()
                              ).getTimeInMillis() / 86400000;
      int daysRemaining = (int) (dateDataset - dateNow);
      int years = yearNow - datasets[i].getYearInt();

      String daysRemainingStr = "";
      if (this.language.equals("de")) {
        if (daysRemaining == 0) {
          daysRemainingStr = "Heute";
        } else if (daysRemaining == 1) {
          daysRemainingStr = "Morgen";
        } else {
          daysRemainingStr = "In "+ daysRemaining +" Tagen";
        }
        returnData[i] =  daysRemainingStr +" ("+ datasets[i].getDate(this.language) +"):  "+
                                                             datasets[i].getText() +" ("+ years +" Jahre)";
      } else if (this.language.equals("en")) {
        if (daysRemaining == 0) {
          daysRemainingStr = "Today";
        } else if (daysRemaining == 1) {
          daysRemainingStr = "Tomorrow";
        } else {
          daysRemainingStr = "In "+ daysRemaining +" days";
        }
        returnData[i] = daysRemainingStr +" ("+ datasets[i].getDate(this.language) +"):  "+
                                                             datasets[i].getText() +" ("+ years +" years)";
      } else {
        if (daysRemaining == 0) {
          daysRemainingStr = "Today";
        } else if (daysRemaining == 1) {
          daysRemainingStr = "Tomorrow";
        } else {
          daysRemainingStr = "In "+ daysRemaining +" days";
        }
        returnData[i] = daysRemainingStr +" ("+ datasets[i].getDate(this.language) +"):  "+
                                                             datasets[i].getText() +" ("+ years +" years)";
      }

      datasetOrder[i] = daysRemaining;
    }

    // sort by month, day (Bubblesort, cause normally the array is very small)
    int hInt = 0;
    String hStr = "";
    for (int i = 0; i < datasetOrder.length-1; i++) {
      if (datasetOrder[i] > datasetOrder[i+1]) {
        hInt = datasetOrder[i+1];
        datasetOrder[i+1] = datasetOrder[i];
        datasetOrder[i] = hInt;
        hStr = returnData[i+1];
        returnData[i+1] = returnData[i];
        returnData[i] = hStr;
      }
    }

    return returnData;
  }


  public int getDayNow() {
    return calendarNowH.get(GregorianCalendar.DATE);
  }

  public int getMonthNow() {
    return calendarNowH.get(GregorianCalendar.MONTH);
  }

  public int getYearNow() {
    return calendarNowH.get(GregorianCalendar.YEAR);
  }

  public String getDateLong() {
    if (language.equals("de")) {
      return "Heute ist der "+ getDayNow() +"."+ getMonthNow() +"."+ getYearNow();
    } else if (language.equals("en")) {
      return "Today is "+ getMonthNow() +"/"+ getDayNow() +"/"+ getYearNow();
    } else {
      return "Today is "+ getYearNow() +"-"+ getMonthNow() +"-"+ getDayNow();    /// TODO: "0" -> "00", "1" -> "01", ...
    }
  }

  public String getLanguage() {
    return this.language;
  }
}